/* @(#)analyse.c	1.16 for x.c 11/23/95 */

#include <stdio.h>
#include <time.h>
#include <math.h>
#include <string.h>
#include <ctype.h>
#ifdef __TURBOC__
#include <alloc.h>
#include <math.h>
#else
#include <malloc.h>
#include <sys/types.h>
#endif

#include "config.h"
#include "globals.h"
#include "x_utils.h"

int destroy_sib(c)
CHILD *c; {

	free(c->fchrom);
	free(c->mchrom);

	return 0;
}

int destroy_family(f)
FAMILY *f; {
	CHILD *c;

	c = f->sibs;
	while (c) {
		destroy_sib(c);

		c = c->next;
	}
	return 0;
}

int prune_families() {
	FAMILY *f, *f0;

	sprintf(msg,"\n----------------------------------- Pruning families\n\n");
	putlog(msg,LOG_NOTICE);
	f = fams; f0 = NULL;
	while (f) {
		if (f->like < fam_like_tol || !f->has_gp) {
			sprintf(msg,"Pruning family %d, L = %f, %s grandparents\n", f->id,f->like,f->has_gp ? "with" : "no");
			putlog(msg,LOG_INFO);
			if (f0 == NULL)
				fams = f->next;
			else
				f0->next = f->next;
			destroy_family(f);
		} else
			f0 = f;
		f = f->next;
	}
/*
	f = fams;
	while (f) {
		sprintf(msg,"Kept family %d, Phase likelihood %5.3f\n", f->id,f->like);
		putlog(msg,LOG_INFO);
		f = f->next;
	}
*/
	return 0;
}

char *fixup_chrom(s)
char *s; {
	char *p, *q = line;
	if (s) 
		for (p = s + 6; *p; p++)
			if (!isspace(*p))
				*q++ = *p;
	*q = '\0';
	free(s);
	return strdup(line);
}

int count_informative(s)
char *s; {
	int i = 0;
	char *p;

	for (p = s; *p; p++)
		if (*p != '-')
			++i;
	return i;
}

float type_per_cM(inf,length)
int inf;
float length; {
	return 10.0 * (float) inf / length;
}

int prune_sibs() {
	FAMILY *f;
	CHILD *c, *c0;
	char *cp;
	int mtypes, ftypes;
	float mdensity, fdensity;

	sprintf(msg,"\n----------------------------------- Pruning siblings\n\n");
	putlog(msg,LOG_NOTICE);
	f = fams;
	while (f) {
		c = f->sibs; c0 = NULL;
		while (c) {
			cp = strrchr(c->fchrom,' ');
			*cp = '\0';
			c->fcross = strtoi(cp + 1);
			c->fchrom = fixup_chrom(c->fchrom);
			ftypes = count_informative(c->fchrom);
			fdensity = type_per_cM(ftypes,flen);
			cp = strrchr(c->mchrom,' ');
			*cp++ = '\0';
			c->mcross = strtoi(cp + 1);
			c->mchrom = fixup_chrom(c->mchrom);
			mtypes = count_informative(c->mchrom);
			mdensity = type_per_cM(mtypes,mlen);
/*
			sprintf(msg,"%5d-%02d, %d f, %d m crossovers\n", f->id,c->id,c->fcross,c->mcross);
			putlog(msg,LOG_INFO);
			sprintf(msg,"  %d/%d female, %d/%d male typings\n", ftypes, strlen(c->fchrom), mtypes, strlen(c->mchrom));
			putlog(msg,LOG_INFO);
			sprintf(msg,"  %5.1f female, %5.1f male typings/cM\n", type_per_cM(ftypes,flen),type_per_cM(mtypes,mlen));
			putlog(msg,LOG_INFO);
*/
			if (fdensity < typ_per_cm_tol && mdensity < typ_per_cm_tol) {
				sprintf(msg,"Pruning %5d-%02d: %5.1f female, %5.1f male typings/cM\n", f->id,c->id,fdensity,mdensity);
				putlog(msg,LOG_INFO);
				if (c0 == NULL)
					f->sibs = c->next;
				else
					c0->next = c->next;
				destroy_sib(c);
			} else
				c0 = c;
			c = c->next;
		}
		f = f->next;
	}
	sprintf(msg,"\n----------------------------------- Siblings remaining\n\n");
	putlog(msg,LOG_NOTICE);
	f = fams;
	while (f) {
		sprintf(msg,"Family %d:", f->id);
		putlog(msg,LOG_INFO);
		c = f->sibs;
		while (c) {
			sprintf(msg," %5d", c->id);
			putlog(msg,LOG_INFO);

			c = c->next;
		}
		putlog("\n",LOG_INFO);
		f = f->next;
	}
	return 0;
}

int prune_families_again() {
	FAMILY *f,*f0;
	CHILD *c;

	sprintf(msg,"\n----------------------------------- Pruning empty families\n\n");
	putlog(msg,LOG_NOTICE);
	f = fams; f0 = NULL;
	while (f) {
		if (f->sibs == NULL) {
			sprintf(msg,"Pruning empty family %d\n", f->id);
			putlog(msg,LOG_INFO);
			if (f0 == NULL)
				fams = f->next;
			else
				f0->next = f->next;
			destroy_family(f);
		} else
			f0 = f;
		f = f->next;
	}
	sprintf(msg,"\n----------------------------------- Siblings remaining\n\n");
	putlog(msg,LOG_NOTICE);
	f = fams;
	while (f) {
		sprintf(msg,"Family %5d:", f->id);
		putlog(msg,LOG_INFO);
		c = f->sibs;
		while (c) {
			sprintf(msg," %d", c->id);
			putlog(msg,LOG_INFO);

			c = c->next;
		}
		putlog("\n",LOG_INFO);
		f = f->next;
	}
	return 0;
}

int checkhap(s,start,size,fam,ind,sex)
char *s;
int start,size,fam,ind;
char sex;	{
	int i,ones = 0, zeros = 0;

	for (i = 0; i < size; i++) {
		switch (*(s + start - 1 + i)) {
			case 'i':
			case '1': ++ones; break;
			case 'o':
			case '0': ++zeros; break;
			default : break;
		}
	}
	if (ones && zeros) {
		strncpy(line,s + start - 1, size);
		line[size] = '\0';
		sprintf(msg,"    %d-%d-%c:intra-hap_sys recombinant detected: '%s'\n", fam,ind,sex,line);
		putlog(msg,LOG_WARNING);
	for (i = 0; i < size; i++) 
		*(s + start - 1 + i) = '-';
		strncpy(line,s + start - 1, size);
		line[size] = '\0';
		sprintf(msg,"      - fixed: '%s'\n", line);
		putlog(msg,LOG_WARNING);	
	}
	return 0;
}
	

int intra_hap_recombs() {
	MAPELEMENT *m;
	FAMILY *f;
	CHILD *c;
	int i;

	sprintf(msg,"\n----------------------------------- Intra-hap_sys recombinants\n\n");
	putlog(msg,LOG_NOTICE);
	for (i = 0; i < hapcount; i++) {
		sprintf(msg,"Checking hap_sys %s:\n",names[haparray[i]->first]);
		putlog(msg,LOG_INFO);
		m = map;
		while (m && strcmp(m->name,names[haparray[i]->first]))
			m = m->next;
		if (m == NULL) {
			sprintf(msg,"  Hap_sys %s not in map\n", names[haparray[i]->first]);
			putlog(msg,LOG_INFO);
			continue;
		}
		sprintf(msg,"  Index %d, size %d\n",m->ord,haparray[i]->count);
		putlog(msg,LOG_INFO);
		f = fams;
		while (f) {
			c = f->sibs;
			while (c) {
				checkhap(c->fchrom,m->ord, haparray[i]->count,f->id,c->id,'M');
				checkhap(c->mchrom,m->ord, haparray[i]->count,f->id,c->id,'P');
				c = c->next;
			}
			f = f->next;
		}	
	}

	return 0;
}

int sort_haps() {
	HAPSYS *h, *h0;
	MAPELEMENT *m;
	int i;

	sprintf(msg,"\n----------------------------------- Sorting hap_sys[0]\n\n");
	putlog(msg,LOG_NOTICE);

	for (i = 0; i < hapcount; i++) {
		m = map;
		while (m && strcmp(m->name,names[haparray[i]->first]))
			m = m->next;
		if (m == NULL) {
			sprintf(msg,"Hap_sys %s not in map\n", names[haparray[i]->first]);
			putlog(msg,LOG_INFO);
			continue;
		}
		haparray[i]->ord = m->ord;
		h = haps; h0 = NULL;
		while (h && h->ord < m->ord) {
			h0 = h;
			h = h->next;
		}
		if (h0)
			h0->next = haparray[i];
		else
			haps = haparray[i];
		haparray[i]->next = h;
	}
	for (i = 0; i < hap0count; i++) {
		m = map;
		while (m && strcmp(m->name,names[hap0array[i]->first]))
			m = m->next;
		if (m == NULL) {
			sprintf(msg,"Hap_sys0 %s not in map\n", names[hap0array[i]->first]);
			putlog(msg,LOG_INFO);
			continue;
		}
		hap0array[i]->ord = m->ord;
		h = haps; h0 = NULL;
		while (h && h->ord < m->ord) {
			h0 = h;
			h = h->next;
		}
		if (h0)
			h0->next = hap0array[i];
		else
			haps = hap0array[i];
		hap0array[i]->next = h;
	}
	h = haps;
	while (h) {
		sprintf(msg,"  Map index %d, name %s, size %d\n",h->ord,names[h->first],h->count);
		putlog(msg,LOG_INFO);

		h = h->next;
	}
	return 0;
}

char collapse(s,start,size)
char *s;
int start,size;	{
	int i,ones = 0, zeros = 0, eyes = 0, owes = 0;
	MAPELEMENT *m;

	for (i = 0; i < size; i++) {
		switch (*(s + start - 1 + i)) {
			case 'i': ++eyes; break;
			case '1': ++ones; break;
			case 'o': ++owes; break;
			case '0': ++zeros; break;
			default : break;
		}
	}
	if ((ones || eyes) && (zeros || owes)) {
/*		strncpy(line,s + start - 1, size);
		line[size] = '\0';
*/
		sprintf(msg,"    intra-hap_sys recombinant detected\n");
		putlog(msg,LOG_WARNING);
		return '?';
	}
	strncpy(msg,s + start - 1,size); msg[size] = ']'; msg[size +1] = '\0';
	fprintf(hapfile,msg);
	if (ones || zeros)
		for (i = 0; i < size; i++) {
			switch (*(s + start - 1 + i)) {
				case '1': ; /* fall through */
				case '0': m = lookup(start - 1 + i); 
					fprintf(hapfile," %s", m->name);
					break;
				default : break;
			}
		}
	if (eyes || owes)
		for (i = 0; i < size; i++) {
			switch (*(s + start - 1 + i)) {
				case 'i': ; /* fall through */
				case 'o': m = lookup(start - 1 + i); 
					fprintf(hapfile," (%s)", m->name);
					break;
				default : break;
			}
		}
		
	if (ones)
		return '1';
	if (zeros)
		return '0';
	if (owes)
		return 'o';
	if (eyes)
		return 'i';
	return '-';
}

char *collapse_hap(s)
char *s;	{
	HAPSYS *h;
	int i;
	char *p = line;
	MAPELEMENT *m;

	sprintf(msg,"%s:\n", s);
	putlog(msg,LOG_INFO);
	i = 0; h = haps;
	while (*(s + i)) {
		if (h && (i + 1) == h->ord) {
			m = lookup(i);
			fprintf(hapfile,"Hap_sys %s [", m->name);
			*p++ = collapse(s,h->ord,h->count);
			fprintf(hapfile,"\n");
			i += h->count;
			h = h->next;
		} else 
			*p++ = *(s + i++);

	}
	*p = '\0';

	sprintf(msg,"%s:\n", line);
	putlog(msg,LOG_INFO);
	free(s);
	return strdup(line);
}
	

int collapse_haps() {
	FAMILY *f;
	CHILD *c;

	sprintf(msg,"\n----------------------------------- Collapsing hap_sys[0]\n\n");
	putlog(msg,LOG_NOTICE);
	f = fams;
	while (f) {
		c = f->sibs;
		while (c) {
			sprintf(msg,"Collapsing %5d-%02d:\n", f->id,c->id);
			putlog(msg,LOG_INFO);
			fprintf(hapfile,"%5d-%02dM:\n", f->id,c->id);
	 		c->fchrom = collapse_hap(c->fchrom);
			fprintf(hapfile,"%5d-%02dP:\n", f->id,c->id);
			c->mchrom = collapse_hap(c->mchrom);
			c = c->next;
		}
		f = f->next;
	}
	return 0;
}

int collapse_map() {
	MAPELEMENT *m, *m0, *m1, *m2;
	HAPSYS *h;
	int count,i;
	float delta,before,after;

	sprintf(msg,"\n----------------------------------- Collapsing map (redistributing distance proportionally)\n\n");
	putlog(msg,LOG_NOTICE);
	m = map; m0 = m1 = m2 = NULL; h = haps; count = 1;
	while (m) {
		if (h && m->ord == h->ord) {
			m0 = m;
			for (i = 0; i < h->count; i++) {
				m1 = m;
				m = m->next;
			}
			m0->next = m;
			h = h->next;
			if (m == NULL) {	/* fallen off the end of the map */
				m0->mdist = m1->mdist;
				m0->fdist = m1->fdist;
			} else if (m1 && m2) {
				delta = m1->mdist - m0->mdist;
				if (delta > 0.001) {
					before = m0->mdist - m2->mdist; if (before < 0.001) before = 0.001;
					after = m->mdist - m1->mdist; if (after < 0.001) after = 0.001;
					m0->mdist += delta * before / (before + after);
					/* m->mdist += delta * after / (before + after); */
				}
				delta = m1->fdist - m0->fdist;
				if (delta > 0.001) {
					before = m0->fdist - m2->fdist; if (before < 0.001) before = 0.001;
					after = m->fdist - m1->fdist; if (after < 0.001) after = 0.001;
					m0->fdist += delta * before / (before + after);
					/* m->fdist += delta * after / (before + after); */
				}
			}
			m2 = m0;
				
		} else {
			m2 = m;
			m = m->next;

		}
		++count;
	}
	m = map; i = 0; 
	while (m) {
		sprintf(msg,"%3d (%3d) %-12s %5.1f %5.1f\n", ++i, m->ord, m->name, m->fdist, m->mdist);
		putlog(msg,LOG_INFO);
		m = m->next;
	}
	return 0;
}

int find_triple(s,fam,sib,sex)
char *s;
int fam,sib;
char sex;	{
	int i = 0, side, crosses = 0, fixups = 0;
	int sides[50], counts[50], breakpoints[50];
	char *s0 = s;
	MAPELEMENT *m1, *m2;

	while (*s == '-') {
		++s; ++i;
	}
	sides[0] = (*s == '1' || *s == 'i');
	counts[0] = 1;
	breakpoints[0] = i;
	while (*(++s)) {
		++i;
		if (*s != '-') {
			side = (*s == '1' || *s == 'i');
			if (side == sides[crosses])
				++counts[crosses];
			else {
				breakpoints[++crosses] = i;
				sides[crosses] = side;
				counts[crosses] = 1;
			}
		}
	}
	if (crosses > 2) 
		for (i = 1; i <= crosses; i++) 
			if (counts[i-1] == 1 && counts[i] == 1) {
				sprintf(msg,"%5d-%02d-%c    %s: %d\n", fam, sib, sex, pretty(s0), crosses);
				putlog(msg,LOG_INFO);
				m1 = lookup(breakpoints[i-1]);
				m2 = lookup(breakpoints[i]);
				sprintf(msg,"       suspicious recombination: %d (%s) - %d (%s)\n", breakpoints[i-1]+1,m1->name, breakpoints[i]+1,m2->name);
				putlog(msg,LOG_INFO);
				*(s0 + breakpoints[i-1]) = '-';
				*(s0 + breakpoints[i]) = '-';
				++fixups;
				sprintf(msg,"Fixed:        %s: %d\n\n",pretty(s0), crosses - 2 * fixups);
				putlog(msg,LOG_INFO);
			}

	return crosses - 2 * fixups;
}

int triple_recombs() {
	FAMILY *f;
	CHILD *c;

	sprintf(msg,"\n----------------------------------- Triple recombinants\n\n");
	putlog(msg,LOG_NOTICE);
	f = fams;
	while (f) {
		c = f->sibs;
		while (c) {
			c->fcross = find_triple(c->fchrom,f->id,c->id,'M');
			c->mcross = find_triple(c->mchrom,f->id,c->id,'P');
			c = c->next;
		}
		f = f->next;
	}

	return 0;
}

int find_double(s,fam,sib,sex)
char *s;
int fam,sib;
char sex;	{
	int i = 0, side, crosses = 0,printed = 0;
	int sides[50], counts[50], breakpoints[50],last[50],first[50];
	char *s0 = s;
	MAPELEMENT *m, *m0, *m1;
	float distance;
	CANDIDATE *c, *c0, *cc;

	while (*s == '-') {
		++s; ++i;
	}
	sides[0] = (*s == '1' || *s == 'i');
	counts[0] = 1;
	breakpoints[0] = i;
	first[0] = last[0] = i;
	while (*(++s)) {
		++i;
		if (*s != '-') {
			side = (*s == '1' || *s == 'i');
			if (side == sides[crosses]) {
				++counts[crosses];
				last[crosses] = i;
			} else {
				breakpoints[++crosses] = i;
				sides[crosses] = side;
				counts[crosses] = 1;
				first[crosses] = last[crosses] = i;
			}
		}
	}
	for (i = 1; i < crosses; i++) 
		if (counts[i] == 1 && counts[i - 1] > 1 && counts[i + 1] > 1) {
			m = lookup(breakpoints[i]);
			m0 = lookup(last[i-1]);
			m1 = lookup(first[i+1]);
			if (sex == 'M')
				distance = m1->fdist - m0->fdist;
			else
				distance = m1->mdist - m0->mdist;
			if ((cc = (CANDIDATE *) malloc(sizeof(CANDIDATE))) == NULL)
				die(ERRNOMEM,"");
			cc->fam = fam; cc->id = sib; cc->sex = sex;
			cc->map_pos = breakpoints[i];
			cc->flank1 = last[i - 1]; cc->flank2 = first[i + 1];
			cc->distance = distance;
			cc->chrom = s0;
			cc->next = NULL;
			if (sex == 'P') {
				c = mcand; c0 = NULL;
				while (c && c->distance < distance) {
					c0 = c;
					c = c->next;
				}
				if (c0 == NULL)
					mcand = cc;
				else
					c0->next = cc;
				cc->next = c;
			} else {
				c = fcand; c0 = NULL;
				while (c && c->distance < distance) {
					c0 = c;
					c = c->next;
				}
				if (c0 == NULL)
					fcand = cc;
				else
					c0->next = cc;
				cc->next = c;
			}
		}
	return 0;
}

int double_recombs() {
	FAMILY *f;
	CHILD *c;
	MAPELEMENT *m, *m0, *m1;
	CANDIDATE *cc;

	sprintf(msg,"\n----------------------------------- Double recombinants\n\n");
	putlog(msg,LOG_NOTICE);
	f = fams;
	while (f) {
		c = f->sibs;
		while (c) {
			if (c->fcross > 1)
				find_double(c->fchrom,f->id,c->id,'M');
			if (c->mcross > 1) 
				find_double(c->mchrom,f->id,c->id,'P');
			c = c->next;
		}
		f = f->next;
	}
	cc = fcand;
	while (cc) {
		m = lookup(cc->map_pos);
		m0 = lookup(cc->flank1);
		m1 = lookup(cc->flank2);
		sprintf(msg,"%5d-%02d-%c: %s\n", cc->fam, cc->id, cc->sex, cc->chrom);
		putlog(msg,LOG_INFO);
		sprintf(msg,"       %d (%s)", cc->map_pos + 1, m->name);
		putlog(msg,LOG_INFO);
		sprintf(msg," flanked by %d (%s), %d (%s)",cc->flank1+1,m0->name,cc->flank2+1,m1->name);
		putlog(msg,LOG_INFO);
		sprintf(msg," %5.1f %s cM\n",cc->distance,cc->sex == 'M' ? "female" : "male");
		putlog(msg,LOG_INFO);
		cc = cc->next;
	}
	putlog("\n",LOG_INFO);
	cc = mcand;
	while (cc) {
		m = lookup(cc->map_pos);
		m0 = lookup(cc->flank1);
		m1 = lookup(cc->flank2);
		sprintf(msg,"%5d-%02d-%c: %s\n", cc->fam, cc->id, cc->sex, cc->chrom);
		putlog(msg,LOG_INFO);
		sprintf(msg,"       %d (%s)", cc->map_pos + 1, m->name);
		putlog(msg,LOG_INFO);
		sprintf(msg," flanked by %d (%s), %d (%s)",cc->flank1+1,m0->name,cc->flank2+1,m1->name);
		putlog(msg,LOG_INFO);
		sprintf(msg," %5.1f %s cM\n",cc->distance,cc->sex == 'M' ? "female" : "male");
		putlog(msg,LOG_INFO);
		cc = cc->next;
	}
	return 0;
}

int fix_double(s,fam,id,sex,ncross)
char *s;
int fam, id, ncross;
char sex;	{
	CANDIDATE *c;
	float limit;
	int i, crosses = ncross, side, oldside, fixed = 0;

	if (sex == 'M') {
		c = fcand;
		limit = female_map_tol;
	} else {
		c = mcand;
		limit = male_map_tol;
	}
	while (c) {
		while (c && (c->fam != fam || c->id != id || c->distance > limit))
			c = c->next;
		if (c) {
			*(s + c->map_pos) = '?';
			++fixed;
			c = c->next;
		}
	}
	if (fixed) {
		sprintf(msg,"Fixed %5d-%02d-%c: %s: ",fam,id,sex,s);
		putlog(msg,LOG_INFO);
		i = 0; crosses = 0;
		while (*s == '-') {
			++s; ++i;
		}
		oldside = (*s == '1' || *s == 'i');
		while (*(++s)) {
			++i;
			if (*s != '-' && *s != '?') {
				side = (*s == '1' || *s == 'i');
				if (side != oldside) {
					++crosses;
					oldside = side;
				}
			}
		}
		sprintf(msg,"%d\n",crosses);
		putlog(msg,LOG_INFO);
	}
	return crosses;
}
	

int fix_double_recombs() {
	FAMILY *f;
	CHILD *c;

	sprintf(msg,"\n----------------------------------- Fixing double recombinants\n\n");
	putlog(msg,LOG_NOTICE);
	f = fams;
	while (f) {
		c = f->sibs;
		while (c) {
			if (c->fcross > 1)
				c->fcross = fix_double(c->fchrom,f->id,c->id,'M',c->fcross);
			if (c->mcross > 1) 
				c->mcross = fix_double(c->mchrom,f->id,c->id,'P',c->mcross);

			c = c->next;
		}
		f = f->next;
	}
	return 0;
}

int new_chrompic() {
	int i;
	FAMILY *f;
	CHILD *c;
	MAPELEMENT *m;

	f = fams;
	while (f) {
		fprintf(out,"\nFamily %d, phase likelihood %5.3f\n",f->id,f->like);
		c = f->sibs;
		while (c) {
			fprintf(out,"  %2d %s %d\n",c->id,pretty(c->fchrom),c->fcross);
			fprintf(out,"     %s %d\n",pretty(c->mchrom),c->mcross);
			c = c->next;
		}
		f = f->next;
	}
	fprintf(out,"\n\nSex-specific map\n\n");
	m = map; i = 1;
	while (m) {
		fprintf(out,"%3d %-12s %5.1f %5.1f\n", i++, m->name, m->fdist, m->mdist);
	 	
		m = m->next;
	}
	return 0;
}

int find_crossovers(s,fam,id,sex)
char *s;
int fam, id;
char sex;	{

	int i = 0, side, crosses = 0,printed = 0;
	int sides[50], counts[50], breakpoints[50],last[50],first[50];
	char *s0 = s;
	MAPELEMENT *m, *m0, *m1;
	float distance;
	CROSSOVER *c, *c0, *cc;
	int j, pk, puk, pos;
	char *s1;

	while (*s == '-') {
		++s; ++i;
	}
	sides[0] = (*s == '1' || *s == 'i');
	counts[0] = 1;
	breakpoints[0] = i;
	first[0] = last[0] = i;
	while (*(++s)) {
		++i;
		if (*s != '-' && *s != '?') {
			side = (*s == '1' || *s == 'i');
			if (side == sides[crosses]) {
				++counts[crosses];
				last[crosses] = i;
			} else {
				breakpoints[++crosses] = i;
				sides[crosses] = side;
				counts[crosses] = 1;
				first[crosses] = last[crosses] = i;
			}
		}
	}
	for (i = 1; i <= crosses; i++) {
		if (counts[i-1] > 1 && counts[i] > 1) {
/*
			m0 = lookup(last[i-1]);
			m1 = lookup(first[i]);
			sprintf(msg,"%5d-%02d-%c: %d - %d", fam,id,sex,last[i-1] + 1,first[i] + 1);
			putlog(msg,LOG_INFO);
			sprintf(msg," (%s - %s)\n",m0->name, m1->name);
			putlog(msg,LOG_INFO);
*/
			if ((cc = (CROSSOVER *) malloc(sizeof(CROSSOVER))) == NULL)
				die(ERRNOMEM,"");
			cc->fam = fam; cc->id = id; cc->sex = sex;
			cc->left = last[i-1]; cc->right = first[i];
			cc->gap = cc->right - cc->left;
			cc->lcount = counts[i-1]; cc->rcount = counts[i];
			cc->before = (i == 1) ? 0 : first[i-1];
			cc->after = (i == crosses) ? strlen(s0) : last[i];
			cc->chrom = s0; cc->next = NULL;
			cc->leftscore = cc->rightscore = 0; pos = 0;
			s1 = cc->chrom + cc->left; j = cc->left - cc->before + 1;
			if (j > 10)
				j = 10;
			while (j--) {
				pk = *(s1 - pos) == '1' || *(s1 - pos) == '0';
				puk = *(s1 - pos) == 'i' || *(s1 - pos) == 'o';
				if (pk || puk) {
					switch (pos) {
						case 0: cc->leftscore += (int) (250.0 * (puk ? puk_factor : 1.0));
						case 1: cc->leftscore += (int) (100.0 * (puk ? puk_factor : 1.0));
						case 2: cc->leftscore += (int) (50.0 * (puk ? puk_factor : 1.0));
						default: cc->leftscore += (int) (10.0 * (float)(10 - pos) * (puk ? puk_factor : 1.0));
					}
				}
				++pos;
			}
			s1 = cc->chrom + cc->right; j = cc->after - cc->right + 1; pos = 0;
			if (j > 10)
				j = 10;
			while (*(s1 + pos) && j--) {
				pk = *(s1 + pos) == '1' || *(s1 + pos) == '0';
				puk = *(s1 + pos) == 'i' || *(s1 + pos) == 'o';
				if (pk || puk) {
					switch (pos) {
						case 0: cc->rightscore += (int) (250.0 * (puk ? puk_factor : 1.0));
						case 1: cc->rightscore += (int) (100.0 * (puk ? puk_factor : 1.0));
						case 2: cc->rightscore += (int) (50.0 * (puk ? puk_factor : 1.0));
						default: cc->rightscore += (int) (10.0 * (float)(10 - pos) * (puk ? puk_factor : 1.0));
					}
				}
				++pos;
			} 
			cc->score = (cc->leftscore < cc->rightscore) ? cc->leftscore : cc->rightscore;


			c = crossovers; c0 = NULL;
			while (c && rankslower(c,cc)) {
				c0 = c;
				c = c->next;
			}
			if (c0)
				c0->next = cc;
			else
				crossovers = cc;
			cc->next = c;
		}

	}
	return 0;
}

int rankslower(c1, c2) 
CROSSOVER *c1, *c2; {
	if (c1->left < c2->left)
		return 1;
	if (c1->left == c2->left && c1->right < c2->right)
		return 1;
	if (c1->left == c2->left && c1->right == c2->right && c1->score > c2->score)
		return 1;
	return 0;
}


int find_recombs() {
	FAMILY *f;
	CHILD *c;
	MAPELEMENT *m, *m0, *m1;
	CROSSOVER *x;
	char *s1, *s2;
	int i,j,count,pre,post,maxwidth = 0;

	m = map; count = 0;
	while (m) {
		if (strlen(m->name) > maxwidth)
			maxwidth = strlen(m->name);
		++count;
		m = m->next;
	}
	fprintf(dat,"%s\n",MAGIC_COOKIE);
	fprintf(dat,"%d\n",count);
	m = map;
	while (m) {
		fprintf(dat,"%s\n",m->name);
		m = m->next;
	}

	sprintf(msg,"\n----------------------------------- Finding well-characterised recombinants\n\n");
	putlog(msg,LOG_NOTICE);
	f = fams;
	while (f) {
		c = f->sibs;
		while (c) {
			if (c->fcross)
				find_crossovers(c->fchrom,f->id,c->id,'M');
			if (c->mcross)
				find_crossovers(c->mchrom,f->id,c->id,'P');

			c = c->next;
		}
		f = f->next;
	}
	x = crossovers;
	while (x) {
		sprintf(msg,"%5d-%02d-%c: %3d - %3d", x->fam,x->id,x->sex,x->left + 1,x->right + 1);
		putlog(msg,LOG_INFO);
		m0 = lookup(x->left);
		m1 = lookup(x->right);
		sprintf(msg," (%-*s - %*s)", maxwidth, m0->name, maxwidth, m1->name);
		putlog(msg,LOG_INFO);
		j = x->left - x->before + 1;
		s1 = x->chrom + x->before; s2 = line; 
		if (x->before == 0) {
			*s2++ = 'p';
			*s2++ = '<';
		}		
		while(j--) 
			*s2++ = *s1++;
		*s2++ = '/';
		i = x->gap;
		while (--i) 
			*s2++ = *s1++;
		*s2++ = '/';
		j = x->after - x->right + 1;
		while (*s1 && j--)
			*s2++ = *s1++;
		if (*s1 == '\0') {
			*s2++ = '>';
			*s2++ = 'q';
		}
		*s2 = '\0';


		sprintf(msg," %4d/%4d : %s\n",x->leftscore,x->rightscore,line);
		putlog(msg,LOG_INFO);
		s1 = line;
		if (*(s1 + strlen(line) - 1) == 'q')
			*(s1 + strlen(line) - 2) = '\0';
		if (*s1 == 'p')
			s1 += 2;
		pre = 0;
		s2 = s1;
		while (*s2++ == '-') {
			++pre;
		}
		s2 = s1 + strlen(s1) - 1;
		if (x->after >= count)
			x->after = count - 1;
		post = count - 1;
		while (*s2-- == '-') {
			--post;
		}
		if (x->leftscore < minscore || x->rightscore < minscore)
			fprintf(dat, "#");
		fprintf(dat,"%5d %2d %c %3d %3d %3d %3d %3d %3d %4d %4d %s\n", x->fam, x->id, x->sex, x->before, pre, x->left, x->right, post, x->after, x->leftscore, x->rightscore, s1);
		x = x->next;
	}
	return 0;
}

int analyse() {
	prune_families();
	prune_sibs();
	prune_families_again();
	intra_hap_recombs();
	sort_haps();
	collapse_haps();
	collapse_map();
	triple_recombs();
	double_recombs();
	fix_double_recombs();
	find_recombs();

	new_chrompic();

	return 0;
}
