/*  @(#)bpt2ps.c	1.16 11/23/95 */

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <malloc.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "config.h"
#define __MAIN__
#include "globals.h"
#include "x_utils.h"
#include "rcparse.h"

#undef RCFILE
#undef LOGFILE
#define INCH 72.0

#define PSFILE "x.ps"
#define RCFILE "bpt2ps.rc"
#define LOGFILE "bpt2ps.log"
#define TOPMARGIN 0.5
#define BOTTOMMARGIN 0.5
#define SIDEMARGIN 0.5
#define PAGEHEIGHT 8.25 
#define PAGEWIDTH 11.65
#define CROPLINE 0.2
#define LINEHEIGHT 8
#define DEBUGLEVEL 0
#define USE_LANDSCAPE 1
#define TOP_LABEL 0
#define SIDE_LABEL 0

#ifndef NAMESX
#define NAMESX (15 * LINEHEIGHT / INCH)
#endif
#ifndef NAMESY
#define NAMESY (4.5 * LINEHEIGHT / INCH)
#endif

static char SccsId_bpt2ps[] = "@(#)bpt2ps.c	1.16\t11/23/95";

typedef struct rcd {
	int 	fam, id;
	char	 sex;
	int	before, pre, left,
		right, post, after,
		lscore, rscore;
	char	*chrom;
	struct rcd *next;
} RCD;

FILE *in, *ps;
char line[LINEBUFSIZE], *rundate;
int numloc, numchrom, rowbreaks[100],maxlen = 0, dbglvl = DEBUGLEVEL, use_landscape = USE_LANDSCAPE,
	side_label = SIDE_LABEL, top_label = TOP_LABEL;
RCD *rcdlist = NULL;
float top = TOPMARGIN * INCH, bottom = BOTTOMMARGIN * INCH, 
	side = SIDEMARGIN * INCH, height = PAGEHEIGHT * INCH, 
	width = PAGEWIDTH * INCH, namesx = NAMESX * INCH, 
	namesy = NAMESY * INCH, lineh = LINEHEIGHT, crop = CROPLINE * INCH;

char *fgetline() {
	do {
		if (feof(in)) 
			die(ERREOF,"");
		fgets(line,LINEBUFSIZE,in);
		line[strlen(line) - 1] = '\0';
	} while (line[1] == '#');
	return line;
}

int psheader() {
	time_t t;

	t = time((time_t *) NULL);
	fprintf(ps,"%%!PS-Adobe-2.0\n");
	fprintf(ps,"%%%%DocumentFonts: Helvetica\n");
	fprintf(ps,"%%%%Title: Breakpoint map\n");
	fprintf(ps,"%%%%Creator: bpt2ps v%s\n",VERSION);
	fprintf(ps,"%%%%CreationDate: %s", ctime(&t));

/*	fprintf(ps,"%%%%BoundingBox: 42 28 552 807\n");
*/
	fprintf(ps,"%%%%EndComments\n");
	return 0;
}

int pstrailer() {
	fprintf(ps,"\n\n%%%%Trailer\n");
	return 0;
}

int psdefs() {
	int i;
	char *s;

	fprintf(ps,"%% PaperWidth   = %5.2f\n",width);
	fprintf(ps,"%% PaperHeight  = %5.2f\n",height);
	fprintf(ps,"%% LineHeight   = %5.2f\n",lineh);
	fprintf(ps,"%% SideMargin   = %5.2f\n",side);
	fprintf(ps,"%% TopMargin    = %5.2f\n",top);
	fprintf(ps,"%% BottomMargin = %5.2f\n",bottom);
	fprintf(ps,"%% CropLine     = %5.2f\n",crop);
	fprintf(ps,"%% NamesX       = %5.2f\n",namesx);
	fprintf(ps,"%% NamesY       = %5.2f\n",namesy);

	return 0;
}

int getdata() {
	int fam, id;
	char sex;
	int before, after, left, right, pre, post;
	int lscore, rscore, start, i;
	char buf[LINEBUFSIZE], **names;
	RCD *r, *r1 = NULL;

	fgetline(); sscanf(line," %d",&numloc);
	fprintf(ps,"\n/NumLoc %d def\n", numloc);
	fprintf(ps,"\n%% locus names\n");
	fprintf(ps,"/names[");
	for (i = 0; i < numloc; i++) {
		fprintf(ps,"(%s)",fgetline());
		if (strlen(line) > maxlen)
			maxlen = strlen(line);
		if (i && i % 5 == 0)
			fprintf(ps,"\n");
	}
	fprintf(ps,"] def\n");
	namesx = maxlen * lineh;
	numchrom = 0;
	while (!feof(in)) {
		do {
			fgetline();
		} while (!feof(in) && line[0] == '#');
		if (feof(in))
			break;
		sscanf(line," %d %d %c %d %d %d %d %d %d %d %d %s", &fam, &id, &sex, &before, &pre, &left, &right, &post, &after, &lscore, &rscore, buf);
		if (after >= numloc)
			after = numloc - 1;
		if ((r = (RCD *) malloc(sizeof(RCD))) == NULL) 
			die(ERRNOMEM,"");
		if ((r->chrom = strdup(buf)) == NULL) 
			die(ERRNOMEM,"");
		r->fam = fam; r->id = id; r->sex = sex;
		r->before = before; r->pre = pre; r->left = left;
		r->right = right; r->post = post; r->after = after;
		r->lscore = lscore; r->rscore = rscore;
		r->next = NULL;
		if (r1 == NULL)
			rcdlist = r;
		else
			r1->next = r;
		r1 = r;
		++numchrom;
	}
	return 0;
}

int utils() {
	time_t t;

	fprintf(ps,"/fs { exch findfont exch scalefont setfont } def\n");
	fprintf(ps,"/rshow { dup stringwidth pop neg 0 rmoveto show } def\n");
	fprintf(ps,"/cshow { dup stringwidth pop 2 div neg 0 rmoveto show } def\n");
	fprintf(ps,"/ashow {gsave moveto 45 rotate /Helvetica %d fs show grestore } def\n",(int) (lineh - 1.0));
	fprintf(ps,"/LandScape { %5.2f 0 translate 90 rotate } def\n", height);
	fprintf(ps,"\n");
	fprintf(ps,"/PageInit {\n");
	if (use_landscape)
		fprintf(ps,"\tLandScape\n");
	fprintf(ps,"\t/Helvetica 6 fs\n");
	fprintf(ps,"\t%5.2f %5.2f moveto\n", side + 5.0, bottom - 7.0);
	fprintf(ps,"\t(Page ) show show\n");
	fprintf(ps,"\t%5.2f %5.2f moveto\n", width / 2, bottom - 7.0);
	fprintf(ps,"\t(Breakpoint data last modified %s) cshow",rundate);
	t = time((time_t *) NULL);
	fprintf(ps,"\t%5.2f %5.2f moveto\n", width - side - 5.0, bottom - 7.0);
	fprintf(ps,"\t(PostScript created %s) rshow",ctime(&t));
	fprintf(ps,"\t/Helvetica %d fs\n", (int)(lineh - 3.0));
	fprintf(ps,"\t0 setlinewidth\n");
	fprintf(ps,"%% lower left\n");
	fprintf(ps,"\t%5.2f %5.2f moveto\n", side - crop, bottom);
	fprintf(ps,"\t%5.2f 0 rlineto stroke\n", crop);
	fprintf(ps,"\t%5.2f %5.2f moveto\n",side, bottom - crop);
	fprintf(ps,"\t0 %5.2f rlineto stroke\n", crop);
	fprintf(ps,"%% upper left\n");
	fprintf(ps,"\t%5.2f %5.2f moveto\n", side - crop, height - top);
	fprintf(ps,"\t%5.2f 0 rlineto stroke\n", crop);
	fprintf(ps,"\t%5.2f %5.2f moveto\n", side, height - top + crop);
	fprintf(ps,"\t0 %5.2f neg rlineto stroke\n", crop);
	fprintf(ps,"%% lower right\n");
	fprintf(ps,"\t%5.2f %5.2f moveto\n", width - side + crop, bottom);
	fprintf(ps,"\t%5.2f 0 rlineto stroke\n", -crop);
	fprintf(ps,"\t%5.2f %5.2f moveto\n", width - side, bottom -crop);
	fprintf(ps,"\t0 %5.2f rlineto stroke\n", crop);
	fprintf(ps,"%% upper right\n");
	fprintf(ps,"\t%5.2f %5.2f moveto\n", width - side + crop, height - top);
	fprintf(ps,"\t%5.2f 0 rlineto stroke\n", -crop);
	fprintf(ps,"\t%5.2f %5.2f moveto\n", width - side, height - top + crop);
	fprintf(ps,"\t0 %5.2f rlineto stroke\n", -crop);
	fprintf(ps,"%% clip rectangle\n");
	fprintf(ps,"\t%5.2f %5.2f moveto\n", side, bottom);
	fprintf(ps,"\t%5.2f %5.2f lineto\n", side, height - (top_label ? 0 : top));
	fprintf(ps,"\t%5.2f %5.2f lineto\n", width - (side_label ? 0 : side), height - (top_label ? 0 : top));
	fprintf(ps,"\t%5.2f %5.2f lineto\n", width - (side_label ? 0 : side), bottom);
	fprintf(ps,"\tclosepath clip newpath\n");
	fprintf(ps,"} def\n");
	fprintf(ps,"\n");
	fprintf(ps,"/SquareFill {\t\t%% stack = grey ---\n");
	fprintf(ps,"\tcurrentpoint\n");
	fprintf(ps,"\tgsave\n");
	fprintf(ps,"\tnewpath\n");
	fprintf(ps,"\tmoveto\n");
	fprintf(ps,"\t%5.2f 0 rlineto\n", lineh - 2.0);
	fprintf(ps,"\t0 %5.2f rlineto\n", lineh - 2.0);
	fprintf(ps,"\t%5.2f 0 rlineto\n", -(lineh - 2.0));
	fprintf(ps,"\tclosepath\n");
	fprintf(ps,"\tgsave setgray fill grestore\n");
	fprintf(ps,"\t0 setgray stroke\n");
	fprintf(ps,"\tgrestore\n");
	fprintf(ps,"} def\n");
	fprintf(ps,"\n");
	fprintf(ps,"/SquareStroke {\t\t%% stack = ---\n");
	fprintf(ps,"\tcurrentpoint\n");
	fprintf(ps,"\tgsave\n");
	fprintf(ps,"\tnewpath\n");
	fprintf(ps,"\tmoveto\n");
	fprintf(ps,"\t%5.2f 0 rlineto\n", lineh - 2.0);
	fprintf(ps,"\t0 %5.2f rlineto\n", lineh - 2.0);
	fprintf(ps,"\t%5.2f 0 rlineto\n", -(lineh - 2.0));
	fprintf(ps,"\tclosepath\n");
	fprintf(ps,"\t0 setgray stroke\n");
	fprintf(ps,"\tgrestore\n");
	fprintf(ps,"} def\n");
	fprintf(ps,"\n");
	fprintf(ps,"/SquareShow {\n");
	fprintf(ps,"\tgsave %5.2f 1 rmoveto setgray cshow grestore\n", (lineh - 2.0) / 2.0);
	fprintf(ps,"} def\n");
	fprintf(ps,"\n");
	fprintf(ps,"/A {\t\t\t%% Open square\n");
	fprintf(ps,"\tSquareStroke\n");
	fprintf(ps,"} def\n");
	fprintf(ps,"/B {\t\t\t%% Open square with 'o'\n");
	fprintf(ps,"\tSquareStroke\n");
	fprintf(ps,"\t(o) 0 SquareShow\n");
	fprintf(ps,"} def\n");
	fprintf(ps,"/C {\t\t\t%% Open square with '0'\n");
	fprintf(ps,"\tSquareStroke\n");
	fprintf(ps,"\t(0) 0 SquareShow\n");
	fprintf(ps,"} def\n");
	fprintf(ps,"/D {\t\t\t%% Open square with '-'\n");
	fprintf(ps,"\tSquareStroke\n");
	fprintf(ps,"\t(-) 0 SquareShow\n");
	fprintf(ps,"} def\n");
	fprintf(ps,"/E {\t\t\t%% Open square with '?'\n");
	fprintf(ps,"\tSquareStroke\n");
	fprintf(ps,"\t(?) 0 SquareShow\n");
	fprintf(ps,"} def\n");
	fprintf(ps,"/F {\t\t\t%% Filled square\n");
	fprintf(ps,"\t0 SquareFill\n");
	fprintf(ps,"} def\n");
	fprintf(ps,"/G {\t\t\t%% Filled square with 'i'\n");
	fprintf(ps,"\t0 SquareFill\n");
	fprintf(ps,"\t(i) 1 SquareShow\n");
	fprintf(ps,"} def\n");
	fprintf(ps,"/H {\t\t\t%% Filled square with '1'\n");
	fprintf(ps,"\t0 SquareFill\n");
	fprintf(ps,"\t(1) 1 SquareShow\n");
	fprintf(ps,"} def\n");
	fprintf(ps,"/I {\t\t\t%% Filled square with '-'\n");
	fprintf(ps,"\t0 SquareFill\n");
	fprintf(ps,"\t(-) 1 SquareShow\n");
	fprintf(ps,"} def\n");
	fprintf(ps,"/J {\t\t\t%% Filled square with '?'\n");
	fprintf(ps,"\t0 SquareFill\n");
	fprintf(ps,"\t(?) 1 SquareShow\n");
	fprintf(ps,"} def\n");
	fprintf(ps,"/K {\t\t\t%% Grey square\n");
	fprintf(ps,"\t0.5 SquareFill\n");
	fprintf(ps,"} def\n");
	fprintf(ps,"\n");
	fprintf(ps,"/.notdef { } def\n");
	fprintf(ps,"\n");
	fprintf(ps,"/Encoding 256 array def\n");
	fprintf(ps,"0 1 255 {\n");
	fprintf(ps,"\tEncoding exch /.notdef put\n");
	fprintf(ps,"} for\n");
	fprintf(ps,"\n");
	fprintf(ps,"Encoding 65 [\n");
	fprintf(ps,"\t/A /B /C /D /E\n");
	fprintf(ps,"\t/F /G /H /I /J\n");
	fprintf(ps,"\t/K\n");
	fprintf(ps,"] putinterval\n");
	fprintf(ps,"\n");
	fprintf(ps,"/gshow {\n");
	fprintf(ps,"\tEncoding exch get\n");
	fprintf(ps,"\tcvx exec\n");
	fprintf(ps,"\t0 %5.2f rmoveto\n", - lineh);
	fprintf(ps,"} def\n");
	fprintf(ps,"\n");
	fprintf(ps,"/gs {\n");
	fprintf(ps,"\tmoveto\n");
	fprintf(ps,"\t{ gshow } forall\n");
	fprintf(ps,"} def\n");
	fprintf(ps,"\n");

	return 0;
}

int saynames() {
	fprintf(ps,"\n%% Write locus names\n");
	fprintf(ps,"/SayNames {\n");
	fprintf(ps,"gsave\n");
	fprintf(ps,"\t/Helvetica %5.2f fs\n", lineh - 1.0);
	fprintf(ps,"\t1 exch {\n");
	fprintf(ps,"\t\tnames exch get rshow\n");
	fprintf(ps,"\t\t0 %5.2f rmoveto\n", - lineh);
	fprintf(ps,"\t} for\n");
	fprintf(ps,"grestore\n");
	fprintf(ps,"} def\n");
	fprintf(ps,"\n");
	
	return 0;
}

char *xlate(r,row)
RCD *r;
int row;	{
	char *s = r->chrom, *s1 = line;
	int i, colour = -1;
	int start = rowbreaks[row] + 1 - r->before, stop = rowbreaks[row + 1] - r->before;
	int pre = r->pre, left = r->left - r->before;
	int post = r->post - r->before, right = r->right - r->before;

	for (i = 0; *s && i <= stop; i++, s++) {
		if (i < pre || i > post || (i > left && i < right))
			colour = -1;
		if (*s == '/') {
			--i;
			continue;
		}
		switch (*s) {
			case 'o':
			case '0': colour = 0; break;
			case 'i':
			case '1': colour = 1; break;
		}
		if (i < start)
			continue;
		switch (*s) {
			case 'o' : *s1++ = (dbglvl >= 2 ? 'B' : 'A'); break;
			case '0' : *s1++ = (dbglvl >= 2 ? 'C' : 'A'); break;
			case 'i' : *s1++ = (dbglvl >= 2 ? 'G' : 'F'); break;
			case '1' : *s1++ = (dbglvl >= 2 ? 'H' : 'F'); break;
			case '-' : switch (colour) {
				case -1: *s1++ = 'K'; break; /* grey */
				case  0: *s1++ = (dbglvl >= 1 ? 'D' : 'A'); break; 
				case  1: *s1++ = (dbglvl >= 1 ? 'I' : 'F'); break;
			} break;
			case '?' : switch (colour) {
				case  0: *s1++ = 'E'; break;
				case  1: *s1++ = 'J'; break;
			} break;
		}
	}
	*s1 = '\0';
	return line;
}

int dopages() {
	int i,j, vert1, vert2, horiz1, horiz2, row = 0, col = 0, page = 0;
	int rows, cols, c; 
	float xpos, yoff, xoff = 2 * lineh;
	RCD *r0, *r = rcdlist;

	horiz1 = (int) ((width - (2 * side) - namesx) / xoff);
	horiz2 = (int) ((width - (2 * side)) / xoff);
	cols = 1; j = numchrom - horiz1;
	if (horiz1 < numchrom)
		while (j > 0) {
			++cols;
			j -= side_label ? horiz1 : horiz2;
		}
	vert1 = (int) ((height - top - bottom - namesy) / lineh);
	vert2 = (int) ((height - top - bottom) / lineh);
	rows = 1; j = numloc - vert1; 
	rowbreaks[0] = -1; rowbreaks[1] = vert1 - 1;
	if (vert1 < numloc)
		while (j > 0) {
			++rows;
			j -= top_label ? vert1 : vert2;
			rowbreaks[rows] = rowbreaks[rows - 1] + (top_label ? vert1 : vert2);
	}
	rowbreaks[rows] = numloc - 1;
	fprintf(ps,"%% rows = %d, cols = %d\n", rows,cols); 
	col = 0; xpos = namesx + lineh; r0 = r;
	for (row = 0; row < rows; row++) {
		r = rcdlist; 
		for (col = 0; col < cols; col++) {
			xpos = side + lineh + ((col == 0 || side_label == 1)? namesx : 0);
			fprintf(ps,"%% row %d; %d to %d\n",row,rowbreaks[row], rowbreaks[row+1]);
			fprintf(ps,"%%%%Page: %c%d %d\n", 'A' + row, col + 1, ++page);
			fprintf(ps,"(%c%d) PageInit\n",'A' + row, col + 1);
			if (col == 0 || side_label == 1) {
				fprintf(ps,"%6.2f %6.2f moveto\n", 
					side + namesx, height - top - lineh - (row == 0 || top_label ? namesy : 0));
				fprintf(ps,"%d %d SayNames\n", rowbreaks[row] + 1, rowbreaks[row+1]);
			} else /* overlap previous page */ {
				if (row == 0 || top_label == 1)
					fprintf(ps,"(%d-%02d%c) %6.2f %6.2f ashow\n", 
						r0->fam,r0->id,r0->sex, xpos - xoff + lineh / 2, height - top - namesy);
			}
			for (c = 0; r && c < ((col == 0 || side_label == 1) ? horiz1 : horiz2); c++) {
				if (row == 0 || top_label == 1)
					fprintf(ps,"(%d-%02d%c) %6.2f %6.2f ashow\n", 
						r->fam,r->id,r->sex, xpos + lineh / 2, height - top - namesy);
				fprintf(ps,"(%s) %6.2f %6.2f gs\n", 
					xlate(r, row), xpos, 
					height - top - ((row == 0 || top_label == 1) ? namesy : 0) - lineh -
					  (lineh * (r->before > rowbreaks[row] + 1 ? r->before - (rowbreaks[row] + 1): 0))
				);
				fprintf(ps,"%% before = %d, rowbreaks[row] = %d\n", r->before, rowbreaks[row]);
				r0 = r; r = r->next;
				xpos += xoff;
			}

			fprintf(ps,"showpage\n");
		}
	}
	return 0;
}


int doit() {
	struct stat buf;

	if ((in = fopen(DATFILE,"rt")) == NULL) 
		die(ERROPEN, DATFILE);
	if ((ps = fopen(PSFILE,"wt")) == NULL) 
		die(ERROPEN, PSFILE);
	fprintf(stderr,"Reading breakpoint data file....\n");
	fgets(line,LINEBUFSIZE,in);
	line[strlen(line) - 1] = '\0';
	if (strcmp(line,MAGIC_COOKIE)) 
		die(ERRBAD,"breakpoint data");
	if (stat(DATFILE,&buf))
		die(ERRBAD,"breakpoint data stat");
	rundate = strdup(ctime(&(buf.st_mtime)));
	psheader();
	getdata();
	fprintf(stderr,"...done\n");
	fprintf(stderr,"Writing PostScript file....\n");
	psdefs();
	utils();
	saynames();
	fprintf(ps,"\n\n");
	dopages();
	pstrailer();
	fprintf(stderr,"...done\n");
	return 0;
}

float getunits() {
	next_tok();
	if (have("point") || have("points")) 
		return 1.0;
	else if (have("inch") || have("inches"))
		return 72.0;
	else if (have("cm"))
		return (72.0 / 2.54);
	else if (have("mm"))
		return (72.0 / 25.4);
	else {
		sprintf(line,"%d of rc file %s:\n\t%s '%s' encountered when expecting one of the following: \n\t\t'point[s]', 'inch[es]', 'cm' or 'mm'\n", rcline,RCFILE,toktypes[toktype],tok);
		die(ERRSYNTAX,line);
	}
	return 0.0;
}

int runcommands() {

	float temp;

	if ((rc = fopen(RCFILE,"rt")) == NULL) {
		die(ERROPEN,RCFILE);
		exit(4);
	}
	fprintf(stderr,"Reading rc file....\n");
	(void) next_tok();
	while (have("set")) {
		if (have("lineheight")) {
			expect(TOK_REAL);
			lineh = realval * getunits();
		} else if (have("topmargin")) {
			expect(TOK_REAL);
			top = realval * getunits();
		} else if (have("bottommargin")) {
			expect(TOK_REAL);
			bottom = realval * getunits();
		} else if (have("sidemargin")) {
			expect(TOK_REAL);
			side = realval * getunits();
		} else if (have("cropline")) {
			expect(TOK_REAL);
			crop = realval * getunits();
		} else if (have("papersize")) {
			if (have("A4")) {
				height = 8.25 * INCH;
				width = 11.65 * INCH;
			} else if (have("A3")) {
				height = 11.65 * INCH;
				width = 16.5 * INCH;
			} else if (have("USletter")) {
				height = 8.5 * INCH;
				width = 11.0 * INCH;
			} else {
			sprintf(line,"%d of rc file %s:\n\tUnrecognised papersize: '%s'\n", rcline,RCFILE,tok);
			die(ERRSYNTAX,line);
			}
		} else if (have("paper_orientation")) {
			if (have("landscape")) {
				use_landscape = 1;
				if (height > width) {
					temp = height; height = width; width = temp;
				}
			} else if (have("portrait")) {
				use_landscape = 0;
				if (width > height) {
					temp = height; height = width; width = temp;
				}
			} else {
			sprintf(line,"%d of rc file %s:\n\tUnrecognised paper_orientation: '%s'\n", rcline,RCFILE,tok);
			die(ERRSYNTAX,line);
			}
		} else if (have("top_label")) {
			if (have("on")) {
				top_label = 1;
			} else if (have("off")) {
				top_label = 0;
			} else {
			sprintf(line,"%d of rc file %s:\n\tUnrecognised top_label option: '%s'\n", rcline,RCFILE,tok);
			die(ERRSYNTAX,line);
			}
		} else if (have("side_label")) {
			if (have("on")) {
				side_label = 1;
			} else if (have("off")) {
				side_label = 0;
			} else {
			sprintf(line,"%d of rc file %s:\n\tUnrecognised side_label option: '%s'\n", rcline,RCFILE,tok);
			die(ERRSYNTAX,line);
			}
		} else if (have("debuglevel")) {
			expect(TOK_INT);
			dbglvl = intval;
			next_tok();
		} else {
			sprintf(line,"%d of rc file %s:\n\tUnrecognised set command: '%s'\n", rcline,RCFILE,tok);
			die(ERRSYNTAX,line);
		}

	}
	if (!have("end")) {
		sprintf(line,"%d of rc file %s:\n\t%s '%s' encountered when keywords 'end' or 'set' expected\n", rcline,RCFILE,toktypes[toktype],tok);
		die(ERRSYNTAX,line);
	}
	fclose(rc);	
	fprintf(stderr,"...done\n");
	return 0;
}

int main() {
	if ((logfile = fopen(LOGFILE,"wt")) == NULL) {
		fprintf(stderr,"Unable to open logfile, aborting program");
		exit(-99);
	}
	runcommands();
	doit();
	return 0;
}
