/*  %W% %G% */

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <malloc.h>

#include "config.h"
#define __MAIN__
#include "globals.h"
#include "x_utils.h"
#include "rcparse.h"

#undef RCFILE
#undef LOGFILE
#define INCH 72.0

#define TXTFILE "x.txt"
#define RCFILE "bpt2txt.rc"
#define LOGFILE "bpt2txt.log"
#define TOPMARGIN 0.5
#define BOTTOMMARGIN 0.5
#define SIDEMARGIN 0.5
#define PAGEHEIGHT 8.25 
#define PAGEWIDTH 11.65
#define CROPLINE 0.2
#define LINEHEIGHT 10
#define DEBUGLEVEL 0
#define USE_LANDSCAPE 0

#ifndef NAMESX
#define NAMESX (15 * LINEHEIGHT / INCH)
#endif
#ifndef NAMESY
#define NAMESY (4.5 * LINEHEIGHT / INCH)
#endif

static char SccsId_bpt2txt[] = "%W%\t%G%";

typedef struct rcd {
	int 	fam, id;
	char	 sex;
	int	before, pre, left,
		right, post, after,
		lscore, rscore;
	char	*chrom;
	struct rcd *next;
} RCD;


FILE *in, *txt;
char line[LINEBUFSIZE], *textlines[500], *locnames[500];
int numloc, numchrom, rowbreaks[100],maxlen = 0, dbglvl = DEBUGLEVEL, use_landscape = USE_LANDSCAPE;
RCD *rcdlist = NULL;
float top = TOPMARGIN * INCH, bottom = BOTTOMMARGIN * INCH, 
	side = SIDEMARGIN * INCH, height = PAGEHEIGHT * INCH, 
	width = PAGEWIDTH * INCH, namesx = NAMESX * INCH, 
	namesy = NAMESY * INCH, lineh = LINEHEIGHT, crop = CROPLINE * INCH;

char *fgetline() {
	do {
		if (feof(in)) 
			die(ERREOF,"");
		fgets(line,LINEBUFSIZE,in);
		line[strlen(line) - 1] = '\0';
	} while (line[1] == '#');
	return line;
}


int getdata() {
	int fam, id;
	char sex;
	int before, after, left, right, pre, post;
	int lscore, rscore, start, i;
	char buf[LINEBUFSIZE], **names;
	RCD *r, *r1 = NULL;

	fgetline(); sscanf(line," %d",&numloc);
	for (i = 0; i < numloc; i++) {
		locnames[i] = strdup(fgetline());
	}
	namesx = maxlen * lineh;
	numchrom = 0;
	while (!feof(in)) {
		do {
			fgetline();
		} while (!feof(in) && line[0] == '#');
		if (feof(in))
			break;
		sscanf(line," %d %d %c %d %d %d %d %d %d %d %d %s", &fam, &id, &sex, &before, &pre, &left, &right, &post, &after, &lscore, &rscore, buf);
		if (after >= numloc)
			after = numloc - 1;
		if ((r = (RCD *) malloc(sizeof(RCD))) == NULL) 
			die(ERRNOMEM,"");
		if ((r->chrom = strdup(buf)) == NULL) 
			die(ERRNOMEM,"");
		r->fam = fam; r->id = id; r->sex = sex;
		r->before = before; r->pre = pre; r->left = left;
		r->right = right; r->post = post; r->after = after;
		r->lscore = lscore; r->rscore = rscore;
		r->next = NULL;
		if (r1 == NULL)
			rcdlist = r;
		else
			r1->next = r;
		r1 = r;
		++numchrom;
	}
	return 0;
}

char *xlate(r,row)
RCD *r;
int row;	{
	char *s = r->chrom, *s1 = line;
	int i, colour = -1;
	int start = rowbreaks[row] + 1 - r->before, stop = rowbreaks[row + 1] - r->before;
	int pre = r->pre, left = r->left - r->before;
	int post = r->post - r->before, right = r->right - r->before;

	for (i = 0; *s && i <= stop; i++, s++) {
		if (i < pre || i > post || (i > left && i < right))
			colour = -1;
		if (*s == '/') {
			--i;
			continue;
		}
		switch (*s) {
			case 'o':
			case '0': colour = 0; break;
			case 'i':
			case '1': colour = 1; break;
		}
		if (i < start)
			continue;
		switch (*s) {
			case 'o' : *s1++ = (dbglvl >= 2 ? 'm' : 'M'); break;
			case '0' : *s1++ = (dbglvl >= 2 ? 'M' : 'M'); break;
			case 'i' : *s1++ = (dbglvl >= 2 ? 'p' : 'P'); break;
			case '1' : *s1++ = (dbglvl >= 2 ? 'P' : 'P'); break;
			case '-' : switch (colour) {
				case -1: *s1++ = '*'; break; /* grey */
				case  0: *s1++ = (dbglvl >= 1 ? '-' : 'M'); break; 
				case  1: *s1++ = (dbglvl >= 1 ? '-' : 'P'); break;
			} break;
			case '?' : switch (colour) {
				case  0: *s1++ = '?'; break;
				case  1: *s1++ = '?'; break;
			} break;
		}
	}
	*s1 = '\0';
	return line;
}

int dopages() {
	int i,j,y,yy,y1, vert1, vert2, horiz1, horiz2, flag, hp, row = 0, col = 0, page = 0;
	int rows, cols, c; 
	float xpos, yoff, xoff = 2 * lineh;
	RCD *r0, *r = rcdlist;
	char *p, buf[20];

/*	horiz1 = (int) ((width - (2 * side) - namesx) / xoff);
	horiz2 = (int) ((width - (2 * side)) / xoff);
*/
	horiz1 = horiz2 = 10;
	cols = 1; j = numchrom - horiz1;
	if (horiz1 < numchrom)
		while (j > 0) {
			++cols;
			j -= horiz2;
		}
/*	vert1 = (int) ((height - top - bottom - namesy) / lineh);
	vert2 = (int) ((height - top - bottom) / lineh);
*/
	vert1 = vert2 = 55;
	rows = 1; j = numloc - vert1; 
	rowbreaks[0] = -1; rowbreaks[1] = vert1 - 1;
	if (vert1 < numloc)
		while (j > 0) {
			++rows;
			j -= vert2;
			rowbreaks[rows] = rowbreaks[rows - 1] + vert2;
	}
	rowbreaks[rows] = numloc - 1;
	for (i = 0; i < vert2 + 25; i++) 
		textlines[i] =  malloc(150);
	col = 0; r0 = r;
	for (row = 0; row < rows; row++) {
		r = rcdlist; 
		for (col = 0; col < cols; col++) {
			hp = 15; yy = 10;
			for (y = 0; y < yy; y++)
				sprintf(textlines[y],"%*s ",hp - 1,"");
			for (y = rowbreaks[row] + 1; y <= rowbreaks[row+1]; y++)
				sprintf(textlines[yy++],"%*s ",hp - 1,locnames[y]);
			sprintf(buf,"Page: %c%d", 'A' + row, col + 1);
			strcat(textlines[3], buf);
			for (c = 0; r && c < (col == 0 ? horiz1 : horiz2); c++) {
				sprintf(buf," %5d",r->fam); 
				strcat(textlines[6], buf);
				sprintf(buf,"    %02d",r->id); 
				strcat(textlines[7], buf);
				sprintf(buf," %5c",r->sex); 
				strcat(textlines[8], buf);
				strcat(textlines[9], "------");

				p = xlate(r,row); flag = 1;
				y1 = -1;
				while (++y1 + rowbreaks[row] + 1 < r->before && y1 < yy) {
					sprintf(buf," %5c", ' ');
					strcat(textlines[y1 + 10],buf);
				}	
				for (y = y1 + 10; y < yy; y++, p++) {
					if (flag)
						flag = *p;
					sprintf(buf," %5c", flag ? *p : ' ');
					strcat(textlines[y],buf);
				}

				r0 = r; r = r->next;
			}
			for (y = 0; y < yy; y++) {
				fprintf(txt,"%s\n",textlines[y]);
			}
			fprintf(txt,"%c",12);

		}
	}
	return 0;
}


int old_dopages() {
	int i,j, vert1, vert2, horiz1, horiz2, row = 0, col = 0, page = 0;
	int rows, cols, c; 
	float xpos, yoff, xoff = 2 * lineh;
	RCD *r0, *r = rcdlist;

	horiz1 = (int) ((width - (2 * side) - namesx) / xoff);
	horiz2 = (int) ((width - (2 * side)) / xoff);
	cols = 1; j = numchrom - horiz1;
	if (horiz1 < numchrom)
		while (j > 0) {
			++cols;
			j -= horiz2;
		}
	vert1 = (int) ((height - top - bottom - namesy) / lineh);
	vert2 = (int) ((height - top - bottom) / lineh);
	rows = 1; j = numloc - vert1; 
	rowbreaks[0] = -1; rowbreaks[1] = vert1 - 1;
	if (vert1 < numloc)
		while (j > 0) {
			++rows;
			j -= vert2;
			rowbreaks[rows] = rowbreaks[rows - 1] + vert2;
	}
	rowbreaks[rows] = numloc - 1;
	fprintf(txt,"%% rows = %d, cols = %d\n", rows,cols); 
	col = 0; xpos = namesx + lineh; r0 = r;
	for (row = 0; row < rows; row++) {
		r = rcdlist; 
		for (col = 0; col < cols; col++) {
			xpos = side + lineh + (col == 0 ? namesx : 0);
			fprintf(txt,"%% row %d; %d to %d\n",row,rowbreaks[row], rowbreaks[row+1]);
			fprintf(txt,"%%%%Page: %c%d %d\n", 'A' + row, col + 1, ++page);
			fprintf(txt,"(%c%d) PageInit\n",'A' + row, col + 1);

			if (col == 0) {

				fprintf(txt,"%6.2f %6.2f moveto\n", 
					side + namesx, height - top - lineh - (row == 0 ? namesy : 0));
				fprintf(txt,"%d %d SayNames\n", rowbreaks[row] + 1, rowbreaks[row+1]);

			} else /* overlap previous page */ {
				if (row == 0)

					fprintf(txt,"(%d-%02d%c) %6.2f %6.2f ashow\n", 
						r0->fam,r0->id,r0->sex, xpos - xoff + lineh / 2, height - top - namesy);

/*
				fprintf(txt,"(%s) %6.2f %6.2f gs\n", 
					xlate(r0, row), xpos, 
					height - top - (row == 0 ? namesy : 0) - lineh -
					  (lineh * (r0->before > rowbreaks[row] + 1 ? r0->before - (rowbreaks[row] + 1): 0))
				);
*/

			}
			for (c = 0; r && c < (col == 0 ? horiz1 : horiz2); c++) {
				if (row == 0)

					fprintf(txt,"(%d-%02d%c) %6.2f %6.2f ashow\n", 
						r->fam,r->id,r->sex, xpos + lineh / 2, height - top - namesy);
				fprintf(txt,"(%s) %6.2f %6.2f gs\n", 
					xlate(r, row), xpos, 
					height - top - (row == 0 ? namesy : 0) - lineh -
					  (lineh * (r->before > rowbreaks[row] + 1 ? r->before - (rowbreaks[row] + 1): 0))
				);
				fprintf(txt,"%% before = %d, rowbreaks[row] = %d\n", r->before, rowbreaks[row]);

				r0 = r; r = r->next;
				xpos += xoff;
			}

			fprintf(txt,"showpage\n");

		}
	}
	return 0;
}


int doit() {

	if ((in = fopen(DATFILE,"rt")) == NULL) 
		die(ERROPEN, DATFILE);
	if ((txt = fopen(TXTFILE,"wt")) == NULL) 
		die(ERROPEN, TXTFILE);
	fprintf(stderr,"Reading breakpoint data file....\n");
	fgets(line,LINEBUFSIZE,in);
	line[strlen(line) - 1] = '\0';
	if (strcmp(line,MAGIC_COOKIE)) 
		die(ERRBAD,"breakpoint data");	
	getdata();
	fprintf(stderr,"...done\n");
	fprintf(stderr,"Writing Ascii text file....\n");
	dopages();
	fprintf(stderr,"...done\n");
	return 0;
}

float getunits() {
	next_tok();
	if (have("point") || have("points")) 
		return 1.0;
	else if (have("inch") || have("inches"))
		return 72.0;
	else if (have("cm"))
		return (72.0 / 2.54);
	else if (have("mm"))
		return (72.0 / 25.4);
	else {
		sprintf(line,"%d of rc file %s:\n\t%s '%s' encountered when expecting one of the following: \n\t\t'point[s]', 'inch[es]', 'cm' or 'mm'\n", rcline,RCFILE,toktypes[toktype],tok);
		die(ERRSYNTAX,line);
	}
	return 0.0;
}

int runcommands() {

	float temp;

	if ((rc = fopen(RCFILE,"rt")) == NULL) {
		die(ERROPEN,RCFILE);
		exit(4);
	}
	fprintf(stderr,"Reading rc file....\n");
	(void) next_tok();
	while (have("set")) {
		if (have("lineheight")) {
			expect(TOK_REAL);
			lineh = realval * getunits();
		} else if (have("topmargin")) {
			expect(TOK_REAL);
			top = realval * getunits();
		} else if (have("bottommargin")) {
			expect(TOK_REAL);
			bottom = realval * getunits();
		} else if (have("sidemargin")) {
			expect(TOK_REAL);
			side = realval * getunits();
		} else if (have("cropline")) {
			expect(TOK_REAL);
			crop = realval * getunits();
		} else if (have("papersize")) {
			if (have("A4")) {
				height = 8.25 * INCH;
				width = 11.65 * INCH;
			} else if (have("A3")) {
				height = 11.65 * INCH;
				width = 16.5 * INCH;
			} else if (have("USletter")) {
				height = 8.5 * INCH;
				width = 11.0 * INCH;
			} else {
			sprintf(line,"%d of rc file %s:\n\tUnrecognised papersize: '%s'\n", rcline,RCFILE,tok);
			die(ERRSYNTAX,line);
			}
		} else if (have("paper_orientation")) {
			if (have("landscape")) {
				use_landscape = 1;
				if (height > width) {
					temp = height; height = width; width = temp;
				}
			} else if (have("portrait")) {
				use_landscape = 0;
				if (width > height) {
					temp = height; height = width; width = temp;
				}
			} else {
			sprintf(line,"%d of rc file %s:\n\tUnrecognised paper_orientation: '%s'\n", rcline,RCFILE,tok);
			die(ERRSYNTAX,line);
			}
		} else if (have("debuglevel")) {
			expect(TOK_INT);
			dbglvl = intval;
			next_tok();
		} else {
			sprintf(line,"%d of rc file %s:\n\tUnrecognised set command: '%s'\n", rcline,RCFILE,tok);
			die(ERRSYNTAX,line);
		}

	}
	if (!have("end")) {
		sprintf(line,"%d of rc file %s:\n\t%s '%s' encountered when keywords 'end' or 'set' expected\n", rcline,RCFILE,toktypes[toktype],tok);
		die(ERRSYNTAX,line);
	}
	fclose(rc);	
	fprintf(stderr,"...done\n");
	return 0;
}

int main() {
	if ((logfile = fopen(LOGFILE,"wt")) == NULL) {
		fprintf(stderr,"Unable to open logfile, aborting program");
		exit(-99);
	}
	runcommands();
	doit();
	return 0;
}
