/* @(#)x.c	1.16 11/23/95 */

#include <stdio.h>
#include <time.h>
#include <math.h>
#include <string.h>
#include <ctype.h>
#include <sys/stat.h>
#ifdef __TURBOC__
#include <alloc.h>
#include <math.h>
#else
#include <malloc.h>
#include <sys/types.h>
#endif

#include "config.h"
#define __MAIN__
#include "globals.h"
#include "x_utils.h"
#include "rcparse.h"

	
int runcommands() {

	if ((rc = fopen(RCFILE,"rt")) == NULL) 
		die(ERROPEN,RCFILE);
	putlog("Reading rc file....\n",LOG_SYSTEM);
	(void) next_tok();
	while (have("set")) {
		if (have("chrompic_file")) {
			expect(TOK_STR);
			chrompic_name = strdup(tok);
			next_tok();
		} else if (have("min_score")) {
			expect(TOK_INT);
			minscore = intval;
			next_tok();
		} else if (have("verbosity")) {
			expect(TOK_INT);
			verbosity = intval;
			next_tok();
		} else if (have("fam_like_tol")) {
			expect(TOK_REAL);
			fam_like_tol = realval;
			next_tok();
		} else if (have("male_map_tol")) {
			expect(TOK_REAL);
			male_map_tol = realval;
			next_tok();
		} else if (have("female_map_tol")) {
			expect(TOK_REAL);
			female_map_tol = realval;
			next_tok();
		} else if (have("min_density")) {
			expect(TOK_REAL);
			typ_per_cm_tol = realval;
			next_tok();
		} else if (have("puk_score_factor")) {
			expect(TOK_REAL);
			puk_factor = realval;
			next_tok();
		} else {
			sprintf(line,"%d of rc file %s:\n\tUnrecognised set command: '%s'\n", rcline,RCFILE,tok);
			die(ERRSYNTAX,line);
		}
	}
	if (!have("end")) {
		sprintf(line,"%d of rc file %s:\n\t%s '%s' encountered when keywords 'end' or 'set' expected\n", rcline,RCFILE,toktypes[toktype],tok);
		die(ERRSYNTAX,line);
	}
	fclose(rc);	
	putlog("...done\n",LOG_SYSTEM);
	return 0;
}


int init() {
	struct stat statbuf;
	time_t t;
	int i;
#ifdef __TURBOC__
	float x;
#endif

	if ((logfile = fopen(LOGFILE,"wt")) == NULL) {
		fprintf(stderr,"Unable to open logfile, aborting program");
		exit(-99);
	}
	runcommands();
#ifdef __TURBOC__
	x = log(6.0);
#endif
	sprintf(msg,"CEPH family breakpoint analysis, version %s\n",VERSION);
	putlog(msg,LOG_SYSTEM);
	t = time((time_t *) NULL);
	sprintf(msg,"%s", ctime(&t));
	putlog(msg,LOG_SYSTEM);
	sprintf(msg,"----------------------------------- Program Initialisation \n");
	putlog(msg,LOG_SYSTEM);
	if ((out = fopen(OUTFILE,"wt")) == NULL) 
		die(ERROPEN,OUTFILE);
	fprintf(out,"Collapsed chrompic file created %s\n", ctime(&t));
	if ((in = fopen(chrompic_name,"rt")) == NULL) 
		die(ERROPEN,chrompic_name);
	if ((dat = fopen(DATFILE,"wt")) == NULL)
		die(ERROPEN,DATFILE);
	if ((hapfile = fopen(HAPFILE,"wt")) == NULL)
		die(ERROPEN,HAPFILE);
	fprintf(hapfile,"Haplotyped systems file created %s\n", ctime(&t));
	setbuf(in,inbuf);
	if (stat(chrompic_name,&statbuf))
		die(ERROPEN,chrompic_name);
	sprintf(msg,"Using chrompic file: %s, last modified %s\n",chrompic_name, ctime(&statbuf.st_mtime));
	fprintf(out,"Using chrompic file: %s, last modified %s\n",chrompic_name, ctime(&statbuf.st_mtime));
	fprintf(hapfile,"Using chrompic file: %s, last modified %s\n",chrompic_name, ctime(&statbuf.st_mtime));
	putlog(msg,LOG_SYSTEM);
	putlog("Internal constants:-\n");
	sprintf(msg,"  Max input line length:            %d\n", LINEBUFSIZE -1);
	putlog(msg,LOG_SYSTEM);
	sprintf(msg,"  Max locus number:                 %d\n", MAXLOC-1);
	putlog(msg,LOG_SYSTEM);
	sprintf(msg,"  Max number of hap_sys:            %d\n", MAXHAP);
	putlog(msg,LOG_SYSTEM);
	sprintf(msg,"  Max number of hap_sys0:           %d\n", MAXHAP);
	putlog(msg,LOG_SYSTEM);
	sprintf(msg,"  Max size of a haplotyped system:  %d\n", MAXHAPSIZE);
	putlog(msg,LOG_SYSTEM);
	sprintf(msg,"  Max number of map points:         %d\n", MAXMAP);
	putlog(msg,LOG_SYSTEM);
	sprintf(msg,"  Min family phase likelihood:      %6.4f\n", fam_like_tol);
	putlog(msg,LOG_SYSTEM);
	sprintf(msg,"  Min typings/10cM maplength:       %-6.2f\n", typ_per_cm_tol);
	putlog(msg,LOG_SYSTEM);
	sprintf(msg,"  Max female flanking distance:     %-6.2f\n", female_map_tol);
	putlog(msg,LOG_SYSTEM);
	sprintf(msg,"  Max male flanking distance:       %-6.2f\n", male_map_tol);
	putlog(msg,LOG_SYSTEM);
	sprintf(msg,"  Min flanking score:               %d\n", minscore);
	putlog(msg,LOG_SYSTEM);
	sprintf(msg,"  PUK score factor:                 %-6.2f\n", puk_factor);
	putlog(msg,LOG_SYSTEM);
	sprintf(msg,"  Reporting verbosity (0 = silent): %d\n", verbosity);
	putlog(msg,LOG_SYSTEM);
	for (i = 0; i < MAXLOC; i++) {
		hapfirst[i] = hapmember[i] = 0;
		names[i] = NULL;
	}
	return 0;
}

int doit() {
	read_chrompic();
	analyse();

	fclose(in);
	fclose(logfile);
	return 0;
}

int main() {
	init();
	doit();
	return 0;
}

